/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * The Original Code is bookmarkshome
 *
 * The Initial Developer of the Original Code is Jeroen Groenenboom.
 *
 * ***** END LICENSE BLOCK ***** */

function bmhHandleFileURL( event ) {
    if ( ! event.isTrusted ) {
	return;
    }
    var isAboutBmh = /^about:((bmh[1-4])|(bookmarkshome)$)/;
    var isFileURL = /^file:\/\//i;
    if( ! event.currentTarget.content ) {
	return;
    }
    var docURL = event.currentTarget.content.location.href;
    var href = event.originalTarget.toString();
    if( isAboutBmh.test( docURL ) && isFileURL.test( href )) {
	bmhHandleLinkClick( event, href, docURL );
    }
}

window.addEventListener( "click", bmhHandleFileURL, true );

function bmhHandleLinkClick( event, href ) {
    var win = event.currentTarget;
    switch ( event.button ) {
	case 0:
	    var external;
	    if( event.ctrlKey ) {
		// sourceURL null, to avoid securitycheck
		win.openNewTabWith( href, null, null, event );
		event.stopPropagation();
	    } 
	    else if( event.shiftKey ) {
		win.openNewWindowWith( href, null, null );
		event.stopPropagation();
	    } 
	    // saving with alt handled by win.handleLinkClick
	    else if(( ! event.metaKey ) && ( ! event.altKey )) {
		// if alt or meta pressed, don't load page
		var is_blank, open_newPref;
		try {
		    // check if it is a _blank link
		    var anchorLis = win.content.document.getElementById( "theRow" )
			.getElementsByTagName( "li" );
		    // there should be at least one link (the one clicked on). 
		    // All links have same target
		    var i = -1;
		    // li might be separator (which has no children)
		    while( ! anchorLis[++i].firstChild );
		    is_blank = ( anchorLis[i].firstChild.getAttribute( "target" ) == "_blank" );
		    open_newPref = win.gPrefService
			.getIntPref( "browser.link.open_newwindow" );
		}
		catch( e ) {
		    open_newPref = 3;
		    is_blank = false;
		}
		// open_new -> 1: same window, 2: new window, 3: new tab 
		if( is_blank ) {
		    if( open_newPref == 2 ) {
			win.openNewWindowWith( href, null, null );
		    }
		    else if( open_newPref == 3 ) {
			win.openNewTabWith( href, null, null, event );
		    }
		}
		else {
		    win.loadURI( href, null, null );
		}
	    }
	    return;
	case 1:         
	    var tab;
	    try {
		tab = win.gPrefService
		    .getBoolPref( "browser.tabs.opentabfor.middleclick" );
	    }
	    catch( e ) {
		tab = true;
	    }
	    if( tab ) {
		win.openNewTabWith( href, null, null, event );
	    }
	    else {
		win.openNewWindowWith( href, null, null );
	    }
	    event.stopPropagation();
	    return;
    }
}



function BmhLib() {
    try {
	// namespaces
	this.NC_NS = "http://home.netscape.com/NC-rdf#";
	this.RDF_NS = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
	this.stylesNS = "http://bookmarkshome.mozdev.org/rdf/styles/";
	this.prefsNS = "http://bookmarkshome.mozdev.org/rdf/prefs/";
	this.pagesNS = "http://bookmarkshome.mozdev.org/rdf/pages/";
	this.accountsNS = "http://bookmarkshome.mozdev.org/rdf/accounts/";
	// types
	this.nsIRDFResource = Components.interfaces.nsIRDFResource;
	this.nsIRDFLiteral =  Components.interfaces.nsIRDFLiteral;
	// services
	this.RDF = Components.classes["@mozilla.org/rdf/rdf-service;1"]
	    .getService( Components.interfaces.nsIRDFService );
	this.RDFC = Components.classes["@mozilla.org/rdf/container;1"]
	    .createInstance( Components.interfaces.nsIRDFContainer );
	this.RDFCU = Components.classes["@mozilla.org/rdf/container-utils;1"]
	    .getService( Components.interfaces.nsIRDFContainerUtils );
	this.ISEARCHSVC = Components.classes["@mozilla.org/rdf/datasource;1?name=internetsearch"]
	    .getService( Components.interfaces.nsIInternetSearchService );
	// arcs 
	this.NameArc = this.RDF.GetResource( this.NC_NS + "Name" );
	this.URLArc =  this.RDF.GetResource( this.NC_NS + "URL" );
	this.IconArc =  this.RDF.GetResource( this.NC_NS + "Icon" );
	this.DescriptionArc = this.RDF.GetResource( this.NC_NS + "Description" );
	this.typeArc = this.RDF.GetResource( this.RDF_NS + "type" );
	this.childArc = this.RDF.GetResource( this.NC_NS + "child" );
	this.FeedURLArc = this.RDF.GetResource( this.NC_NS + "FeedURL" );
	// style arcs
	this.nameArc = this.RDF.GetResource( this.stylesNS + "name" );
	this.sheetArc = this.RDF.GetResource( this.stylesNS + "sheet" );
	// account arcs
	this.hostArc = this.RDF.GetResource( this.prefsNS + "ftpHost" );
	this.userArc = this.RDF.GetResource( this.prefsNS + "ftpUser" );
	this.pathArc = this.RDF.GetResource( this.prefsNS + "ftpPath" );
	// roots
	this.bmRoot = this.RDF.GetResource( "NC:BookmarksRoot" );
	this.rootEngine = this.RDF.GetResource( "NC:SearchEngineRoot" );
	// start bookmarks observer
	this.bmObserver = this.getBookmarksObserver();
	// datasources
	this.BMDS  = this.RDF.GetDataSource( "rdf:bookmarks" );
	this.ISDS = this.RDF.GetDataSource( "rdf:internetsearch" );
	this.stylesDS = this.getDataSource( "styles.rdf" );
	this.prefsDS = this.getDataSource( "bmhPrefs.rdf" );
	// upload
	this.upload = false;
    }
    catch( e ) {
	try {
	    var sbs = Components.classes["@mozilla.org/intl/stringbundle;1"]
		.getService( Components.interfaces.nsIStringBundleService );
	    var strBundle = sbs.createBundle( "chrome://bookmarkshome/locale/bmh.properties" );
	    failStr = strBundle.GetStringFromName( "BmhFailureString" );
	    actionStr = strBundle.GetStringFromName( "BmhLibFailureActionString" );
	    alert( failStr + actionStr );
	}
	catch( e1 ) {
	    alert( e )
	}
	throw( e );
    }
}

BmhLib.prototype = {

getBookmarksObserver: function() {
			  try {
			      // create reference to observer of bookmarks
			      return Components.classes["@jeroen/bmhBookmarksObserverService;1"]
				  .getService( Components.interfaces.nsIRDFObserver );
			  }
			  catch( e ) {
			      dump( e );
			  }
			  try {
			      // something wrong, get dynamic page
			      this.setDynamicsPref( "dynamics", true );
			  }
			  catch( e ) {
			      // if everything goes wrong, bmh page will be
			      // either static or dynamic, no disaster
			      dump( e );
			  }
			  return null;
		      },

getLocaleString: function( strName, formattingValues ) {
		     var str = "???";
		     try {
			 var sbs = Components.classes["@mozilla.org/intl/stringbundle;1"]
			     .getService( Components.interfaces.nsIStringBundleService );
			 var strBundle = sbs.createBundle( "chrome://bookmarkshome/locale/bmh.properties" );
			 if ( arguments.length >= 2 ) {
			     str = strBundle.formatStringFromName( strName, formattingValues, formattingValues.length );
			 } else {
			     str = strBundle.GetStringFromName( strName );
			 }
		     } 
		     catch( e ) {}
		     return str;
		 },

getPrefType: function( name, bStr ) {
		 var branchString = ( arguments.length == 2 ? bStr : "bookmarkshome." );
		 var branch = this.getBranch( branchString ); 
		 return branch.getPrefType( name );
	     },

getPref: function( name, bStr ) {
	     var branchString = ( arguments.length == 2 ? bStr : "bookmarkshome." );
	     var branch = this.getBranch( branchString ); 
	     var type = branch.getPrefType( name );
	     if( type == branch.PREF_INVALID ) {
		 return null;
	     }
	     else if( type == branch.PREF_STRING ) {
		 return branch.getCharPref( name );
	     }
	     else if( type == branch.PREF_INT ) {
		 return branch.getIntPref( name ); 
	     }
	     else if( type == branch.PREF_BOOL ) {
		 return branch.getBoolPref( name );
	     }
	 },

getWhat: function( url ) {
	     if( url == "chrome://bookmarkshome/content/bookmarksHome.html" ) {
		 return "bookmarkshome";
	     }
	     var matchArray = url.match( /^about:((?:bmh[1-4])|(?:bookmarkshome)$)/ );
	     return ( matchArray ? matchArray[1] : url );
	 },


getRDFPref: function( name, url ) {
		try {
		    var what = this.getWhat( url ); 
		    var src = this.RDF.GetResource( this.prefsNS + what );
		    var arc = this.RDF.GetResource( this.prefsNS + name );
		    var target = this.prefsDS.GetTarget( src, arc, true ); 
		    if( target instanceof this.nsIRDFLiteral ) { 
			var prefType = this.getPrefType( name );
			if(( prefType == 0 ) || ( prefType == 32 )) {
			    return target.Value;
			}
			else {
			    return eval( target.Value );
			}
		    }
		}
		catch( e ) {
		    return null;
		}
	    },

openURL: function( url ) {
	     window.getBrowser().selectedBrowser.loadURI( url );
	 },

openInTab: function( url ) {
	     window.getBrowser().loadOneTab( url );
	 },

openReadme: function() {
		var url = this.getLocaleString( "ReadmeHrefString" );
		this.openInTab( url );
	    },

getBranch: function( bStr ) {
	       return Components.classes["@mozilla.org/preferences-service;1"]
		   .getService( Components.interfaces.nsIPrefService )
		   .getBranch( bStr );
	   },

setPref: function( name, value, bStr ) {
	     try {
		 var branchString = ( arguments.length == 3 ? bStr : "bookmarkshome." );
		 var branch = this.getBranch( branchString ); 
		 var type = branch.getPrefType( name );
		 if( type == branch.PREF_STRING ) {
		     branch.setCharPref( name, value );
		 }
		 else if( type == branch.PREF_INT ) {
		     branch.setIntPref( name, value ); 
		 }
		 else if( type == branch.PREF_BOOL ) {
		     branch.setBoolPref( name, value );
		 }
	     }
	     catch( e ) {}
	 },

reloadPage: function() {
		window.getBrowser().selectedBrowser.reload();
	    },

setRDFPref: function( name, value, url ) {
		try {
		    var src = this.RDF.GetResource( this.prefsNS + this.getWhat( url ));
		    var arc = this.RDF.GetResource( this.prefsNS + name );
		    var oldTarget = this.prefsDS.GetTarget( src, arc, true )
			.QueryInterface( this.nsIRDFLiteral );
		    var newTarget = this.RDF.GetLiteral( value );
		    this.prefsDS.Change( src, arc, oldTarget, newTarget ); 
		}
		catch( e ) {
		    alert( this.getLocaleString( "RDFPrefNotSetString", [name,value,url] ));
		}
	    },

clearPref: function( name ) {
	       try {
		   var branch = this.getBranch( "bookmarkshome." );
		   branch.clearUserPref( name );
	       }
	       catch( e ) {}
	   },

getProperty: function ( aInput, aArc, DS ) {
		 try {
		     var node;
		     aInput.QueryInterface( this.nsIRDFResource );
		     node = DS.GetTarget( aInput, aArc, true );
		     if( node instanceof this.nsIRDFResource ) {
			 return node.Value;
		     }
		     if( node instanceof this.nsIRDFLiteral ) {
			 return node.Value;
		     }
		 }
		 catch( e ) {}
		 return "";
	     },

setDynamicsPref: function( url, dynamic ) {
		     // last char of url is number x.
		     var what = this.getWhat( url ); 
		     var digit = parseInt( what == "bookmarkshome" ? "0" : what.charAt( what.length-1 ));
		     if( digit >= 0 ) {
			 // position x in binCheck (l to r, 0-4) is 1, rest is 0
			 var binCheck = 1 << digit;
			 if( dynamic ) {
			     this.setPref( "dynamics", ( binCheck | this.getPref( "dynamics" )));
			 }
			 else {
			     // 5 pages, 31 is binary with 5 ones
			     binCheck = 31 - binCheck;
			     // position x in binCheck is 0, rest is 1 
			     this.setPref( "dynamics", ( binCheck & this.getPref( "dynamics" )));
			 }
		     }
		 },


getBmhFile: function( url ) {
		var what = this.getWhat( url );
		try {
		    // init nsIURL object with chrome url of static page
		    var  bmhStaticLoc = Components.classes["@mozilla.org/network/standard-url;1"]
			.createInstance( Components.interfaces.nsIURL );
		    bmhStaticLoc.spec = "chrome://bookmarkshome/content/" + what + "Saved.html";
		    var  chromeReg = Components.classes["@mozilla.org/chrome/chrome-registry;1"]
			.getService( Components.interfaces.nsIChromeRegistry );
		    // convert chrome url to file url 
		    var fileURL = chromeReg.convertChromeURL( bmhStaticLoc );
		    if( fileURL instanceof Components.interfaces.nsIURI ) {
			// deerpark
			fileURL = fileURL.spec;
		    }
		    // get file from url
		    var bmhFile = Components.classes["@mozilla.org/network/protocol;1?name=file"]
			.createInstance( Components.interfaces.nsIFileProtocolHandler )
			.getFileFromURLSpec( fileURL );
		    return bmhFile;
		}
		catch( e ) {
		    return null;
		}
	    },

getDataSource: function( fileName ) {
		   // let BmhLib fail if datasource cannot be retrieved
		   // get path to profile folder to retrieve/save rdf file
		   var  profileDirString = Components.classes["@mozilla.org/file/directory_service;1"] 
		       .getService( Components.interfaces.nsIProperties )
		       .get( "ProfD", Components.interfaces.nsIFile )
		       .QueryInterface( Components.interfaces.nsILocalFile )
		       .path; 
		   // create rdf file instance and initialize
		   var DSFile = Components.classes["@mozilla.org/file/local;1"].createInstance();
		   if( DSFile instanceof Components.interfaces.nsILocalFile ) {
		       DSFile.initWithPath( profileDirString );
		       DSFile.append( fileName );
		   }
		   // get url of rdf file
		   var DSFileURL = Components.classes["@mozilla.org/network/io-service;1"]
		       .getService( Components.interfaces.nsIIOService )
		       .newFileURI( DSFile )
		       .spec;
		   var dfltDS, DS;
		   if( ! DSFile.exists() ) {
		       // if rdf file does not exist yet, create default
		       dfltDS = this.RDF.GetDataSourceBlocking( "chrome://bookmarkshome/content/" + fileName );
		       dfltDS.QueryInterface( Components.interfaces.nsIRDFRemoteDataSource )
			   .FlushTo( DSFileURL );
		       // get data source 
		       DS = this.RDF.GetDataSourceBlocking( DSFileURL ); 
		       if( fileName == "bmhPrefs.rdf" ) {
			   // add (PrefService)prefs to prefs DS 
			   this.copyPrefsToDS( DS );
		       }
		   }
		   else {
		       // get data source 
		       DS = this.RDF.GetDataSourceBlocking( DSFileURL ); 
		       if( fileName == "styles.rdf" ) {
			   // add newest addons to existing styles file
			   this.addCssToDS( DS, DSFileURL );
		       }
		   }
		   // flush needed if DS was changed
		   try {
		       DS.QueryInterface( Components.interfaces.nsIRDFRemoteDataSource )
			   .Flush();
		   }
		   catch( e ) {}
		   return DS;
	       },

copyPrefsToDS: function( prefsDS ) {
		   try {
		       var bmh0Resource = this.RDF.GetResource( this.prefsNS + "bookmarkshome" );
		       var bmh0Enum = prefsDS.ArcLabelsOut( bmh0Resource );
		       var curArc, curTarget, curPref;
		       while( bmh0Enum.hasMoreElements() ) {
			   curArc = bmh0Enum.getNext().QueryInterface( this.nsIRDFResource );
			   curTarget = prefsDS.GetTarget( bmh0Resource, curArc, true )
			       .QueryInterface( Components.interfaces.nsIRDFLiteral );
			   curPref = this.getPref( curArc.Value.match( /[^\/]+$/ ));
			   if( curPref ) {
			       prefsDS.Change( bmh0Resource, curArc, curTarget, this.RDF.GetLiteral( curPref )); 
			   }
		       }
		   }
		   catch( e ) {
		       // some prefs replaced with defaults
		   }
	       },

addCssToDS: function( DS, DSFileURL ) {
		// let BmhLib fail if addons are not added
		var dfltDS = this.RDF.GetDataSourceBlocking( "chrome://bookmarkshome/content/styles.rdf" );
		var dfltAddonsList = this.RDF.GetResource( this.stylesNS + "addonsList" );
		this.RDFC.Init( dfltDS, dfltAddonsList );
		var enumerator = this.RDFC.GetElements();
		if(( this.getPref( "addon" ) == "addon 140" ) &&
			enumerator.hasMoreElements() ) {
		    enumerator.getNext();
		}
		var curNode, addonTarget;
		var addonsStr = "";
		while( enumerator.hasMoreElements() ) {
		    // retrieve addons
		    curNode = enumerator.getNext()
			.QueryInterface( this.nsIRDFResource );
		    addonTarget = dfltDS.GetTarget( curNode, this.sheetArc, true )
			.QueryInterface( this.nsIRDFLiteral );
		    if( ! DS.HasAssertion( curNode, this.sheetArc, addonTarget, true )) {
			DS.Assert( curNode, this.sheetArc, addonTarget, true ); 	
			addonsStr = ( addonTarget.Value + addonsStr );
		    }
		}
		var stylesList = this.RDF.GetResource( this.stylesNS + "stylesList" );
		this.RDFC.Init( DS, stylesList );
		enumerator = this.RDFC.GetElements();
		var oldTarget, newTarget;
		while( enumerator.hasMoreElements() ) {
		    // add addons
		    curNode = enumerator.getNext().QueryInterface( this.nsIRDFResource );
		    oldTarget = DS.GetTarget( curNode, this.sheetArc, true )
			.QueryInterface( this.nsIRDFLiteral );
		    newTarget = this.RDF.GetLiteral( addonsStr + oldTarget.Value );
		    DS.Change( curNode, this.sheetArc, oldTarget, newTarget );
		}
		this.setPref( "addon", "addon 150" );
	    },

selectEngine: function( dfltOption ) {
		  try {
		      var doc = window.content.document;
		      var searchForm = doc.getElementById( "searchForm" );
		      var engineSelect = doc.getElementById( "engineSelect" );
		      var options = engineSelect.getElementsByTagName( "option" );
		      var engineImg = doc.getElementById( "engineImg" );
		      var firstNotHidden = null;
		      for( var i = options.length - 1; i >= 0; i-- ) {
			  options[i].defaultSelected = false;
			  if( options[i].style.display != "none" ) {
			      firstNotHidden = options[i];
			  }
		      }
		      if(( ! dfltOption ) || ( dfltOption.style.display == "none" )) {
			  if( firstNotHidden ) {
			      // select the first not hidden engine
			      firstNotHidden.selected = true;
			      firstNotHidden.defaultSelected = true;
			      searchForm.style.display = "";
			      engineImg.src = firstNotHidden.value.split( "|" )[1];
			  }
			  else {
			      searchForm.style.display = "none";
			  }
		      }
		      else if( dfltOption ) {
			  // dfltOption is not hidden
			  dfltOption.selected = true;
			  dfltOption.defaultSelected = true;
			  engineImg.src = dfltOption.value.split( "|" )[1];
			  searchForm.style.display = "";
		      }
		  }
		  catch( e ) {}
	      },

testProps: function( object, propChain ) {
	       try {
		   var rv = object;
		   for( var i = 0; i < propChain.length; i++ ) {
		       rv = rv[propChain[i]];
		   }
		   return rv;
	       }
	       catch( e ) {}
	       return null;
	   },

formatSheet: function( sheet ) {
		 // replace all whitespace
		 sheet = sheet.replace( /\s+/g, " " );
		 // newline after ';' or '{' if not (directly) in comment, 
		 // remove possible leading space
		 sheet = sheet.replace( /([\{;]) *([^\* ])/g, "$1\n$2" );
		 // newline after comment, remove possible leading space
		 sheet = sheet.replace( /(\*\/) */g, "$1\n" );
		 // two newlines after '}', remove possible leading space
		 sheet = sheet.replace( /\} */g, "\}\n\n" );
		 // indent properties
		 sheet = sheet.replace( /([^\n;]*;\n)/g, "    $1" );
		 return sheet;
	     },

binarySearch: function( target, array ) {
		  var found = false;
		  var first = 0, last = array.length - 1, temp, mid;
		  while (( !found ) && ( first <= last )) {
		      temp = first + last;
		      // integer division
		      mid = ( temp - ( temp % 2 )) / 2;
		      if( target == array[mid] ) {
			  found = true;
		      }
		      else if( target < array[mid] ) {
			  last = mid - 1;
		      }
		      else {
			  first = mid + 1;
		      }
		  }
		  return( found ? mid : -1 ); 
	      }, 

getArrayFromPref: function( pref ) {
		      var str = this.getPref( pref );
		      var arr = ( str == "" ? [] : str.split( "|" ));
		      for( var i = 0; i < arr.length; i++ ) {
			  arr[i] = decodeURI( arr[i] );
		      }
		      return arr;
		  },

getArrayFromRDFPref: function( pref, url ) {
			 var str = this.getRDFPref( pref, url );
			 if( str == null ) {
			     return [];
			 }
			 var arr = ( str == "" ? [] : str.split( "|" ));
			 for( var i = 0; i < arr.length; i++ ) {
			     arr[i] = decodeURI( arr[i] );
			 }
			 return arr;
		     },

getPrefFromArray: function( arr ) {
		      var pref = "";
		      for ( var i = 0; i < arr.length; i++ ) {
			  pref += ( "|" + encodeURI( arr[i] ));
		      }
		      if( pref.length > 0 ) {
			  // remove first bar
			  pref = pref.substr( 1 );
		      }
		      return pref;
		  },

getRDFPrefFromArray: function( arr ) {
			 if( ! arr ) {
			     return null;
			 }
			 var pref = "";
			 for ( var i = 0; i < arr.length; i++ ) {
			     pref += ( "|" + encodeURI( arr[i] ));
			 }
			 if( pref.length > 0 ) {
			     // remove first bar
			     pref = pref.substr( 1 );
			 }
			 return pref;
		     },

isExcludedFolder: function( folderString, excludeIndex, inOrExcludedFolders ) {
		      var index = this.binarySearch( folderString, inOrExcludedFolders ); 
		      var isInOrExcluded = ( index >= 0 );
		      var exclude = ( excludeIndex == 0 ); 
		      return ( exclude == isInOrExcluded );
		  },

getCurContentDocument: function() {
		    return window.content.document;
		},

getCurContentURL: function() {
		   return window.content.location.href;
	       },

getNavigatorDocument: function() {
			     return window.document;
			 },
}

var bmhLib = new BmhLib();
